package hdctl.net
{
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.ProgressEvent;
	import flash.net.Socket;
	import flash.net.URLVariables;
	import flash.utils.ByteArray;
	
	[Event(name="close", type="flash.events.Event")]
		
	public class HTTPServer extends EventDispatcher
	{
		private var File:Class;
		private var FileStream:Class;
		private var rrdgraph_cn_async:Function;
		
		private var program:Object;
		private var server:Object;
		private var port:uint;
		private var docroot:String;

		public var parse_files:RegExp = /^index\.html$/i;//	/\.(html?|css|js|xml|txt)$/i
		
		public function HTTPServer(program:Object, docroot:String, port:uint = 80)
		{
			super(null);

			this.program = program;
			this.docroot = docroot;
			this.port = port;
			
			var ServerSocket:Object = program.getDefinition('flash.net::ServerSocket');
			server = new ServerSocket;
			server.bind(port);
			if (server.bound) 
			{
				log('bound to port ' + port + ' on ' + server.localAddress);
				server.listen();
				if (server.listening) 
				{
					log('listening for incoming connections');
					server.addEventListener(Event.CLOSE, socket_close);
					server.addEventListener(Event.CONNECT, client_connect);
					
					File = program.getDefinition('flash.filesystem::File') as Class;
					FileStream = program.getDefinition('flash.filesystem::FileStream') as Class;
					rrdgraph_cn_async = program.getDefinition('rrdtool.util::rrdgraph_cn_async') as Function;
				}
				else log('cant listen to incoming connectoions on ' + server.localAddress + ':' + port);
			}
			else log('cant bind to port ' + port + ' on ' + server.localAddress);
		}
		
		public function close () : void
		{
			server.close();
		}
		
		private function readfile (file:Object) : ByteArray
		{
			var fs:Object = new FileStream,
				out:ByteArray = new ByteArray;
			fs.open(file, 'read');
			fs.readBytes(out);
			fs.close();
			return out;
		}
		
		private function socket_close(event:Event) : void
		{
			server.removeEventListener(Event.CONNECT, client_connect);
			server = null;
			log('socket closed by OS');
			program = null;
			dispatchEvent(event);
		}
		
		private function client_connect (event:Object) : void 
		{
			event.socket.addEventListener(ProgressEvent.SOCKET_DATA, socket_data);
		}
		
		private function log(...args:Array) : void
		{
			program.print('HTTPServer: ' + args);
		}

		private function socket_data(event:ProgressEvent):void
		{
			var client:Object = event.target,
				input:String = client.readUTFBytes(client.bytesAvailable),
				request:Array = input.match(/^GET\s([^\s]+)\sHTTP\/1\.1/);
			client.removeEventListener(ProgressEvent.SOCKET_DATA, socket_data);
			
			//log('[' + client.remoteAddress + ':' + client.remotePort + '] ' + request[1]);
			
			if (request is Array && request.length >= 2) 
			{
				request = request[1].split('?');
				var path:String = request[0];
				var args:URLVariables = new URLVariables(request[1]);
				var file:Object  = new File(docroot + path);

				if (file.exists) 
				{
					if (file.isDirectory)
					{
						var index:Object = new File(file.nativePath + File.separator + 'index.html');
						if (index.exists) 
						{
							client.writeUTFBytes('HTTP/1.1 301 Moved Permanently\n');
							client.writeUTFBytes('Location: /index.html' + (request[1] ? '?' + args : '') + '\n\n');
						}
						else
						{
							client.writeUTFBytes('HTTP/1.1 403 Forbidden\n');
							client.writeUTFBytes('Content-Type: text/html\n');
							client.writeUTFBytes('Connection: close\n\n');
							client.writeUTFBytes('<h1>403 Forbidden</h1>\n');
							client.writeUTFBytes('<p>Directory index not implemented yet.</p>\n');
						}
					}
					else
					{
						client.writeUTFBytes('HTTP/1.1 200 OK\n');
						switch (file.extension)
						{
							case 'htm':
							case 'html': client.writeUTFBytes('Content-Type: text/html\n'); break;
							case 'xml': client.writeUTFBytes('Content-Type: text/xml\n'); break;
							case 'js': client.writeUTFBytes('Content-Type: application/javascript\n'); break;
							case 'css': client.writeUTFBytes('Content-Type: text/stylesheet\n'); break;
							case 'txt': client.writeUTFBytes('Content-Type: text/plain\n'); break;
							case 'jpg':
							case 'jpeg': client.writeUTFBytes('Content-Type: image/jpeg\n'); break;
							case 'gif': client.writeUTFBytes('Content-Type: image/gif\n'); break;
							case 'png': client.writeUTFBytes('Content-Type: image/png\n'); break;
							default : 
								client.writeUTFBytes('Content-Type: application/octet-stream\n');
								client.writeUTFBytes('Content-Disposition: attachement; filename="' + file.name + '"\n');
						}
						client.writeUTFBytes('Connection: close\n');
						
						if (program.hasOwnProperty('parser') && program.parser is Function && file.name.match(parse_files)) 
						{
							var fs:Object = new FileStream;
							fs.open(file, 'read');
							var src:String = fs.readUTFBytes(fs.bytesAvailable);
							fs.close();
							try {
								src = program.parser(src, file, args);
							} catch (err:Error) {
								log('invalid parser implementation: ' + err);
							}
							var bin:ByteArray = new ByteArray;
							bin.writeUTFBytes(src);
							client.writeUTFBytes('Cache-Control: no-cache, must-revalidate\n');
							client.writeUTFBytes('Content-Length: ' + bin.length + '\n\n');
							client.writeBytes(bin);
						}
						else 
						{
							client.writeUTFBytes('Content-Length: ' + file.size + '\n\n');
							client.writeBytes(readfile(file));
						}
					}
				}
				else if (file.name == 'graph.png') // reserved filename
				{
					if (!args.conf || isNaN(Number(args.start)) || isNaN(Number(args.end)) || isNaN(Number(args.width)) || isNaN(Number(args.height)))
						log('invalid argumnets: ' + args);
					else 
					{						
						try {
							rrdgraph_cn_async(args.conf, args.start, args.end, args.width, args.height, graph_output, client);
							return;
						} catch (error:Error) {
							if (error.errorID == 1) log('нет такой конфигурации графика');
							if (error.errorID == 2) log('нет ни одного DS [с включенным логом] в конфигурации графика');
						}
					}
				}
				else
				{
					client.writeUTFBytes('HTTP/1.1 404 Not Found\n');
					client.writeUTFBytes('Content-Type: text/html\n');
					client.writeUTFBytes('Connection: close\n\n');
					client.writeUTFBytes('<h1>404 Not Found</h1>\n');
					client.writeUTFBytes('<p>' + file.name + ' does not exists.</p>\n');
				}
				client.flush();
			}
			client.close();
			client = null;
		}
		
		public function graph_output (png:ByteArray, client:Socket) : void
		{
			client.writeUTFBytes('HTTP/1.1 200 OK\n');
			client.writeUTFBytes('Cache-Control: no-cache, must-revalidate\n');
			client.writeUTFBytes('Content-Type: image/png\n');
			client.writeUTFBytes('Connection: close\n');
			client.writeUTFBytes('Content-Length: ' + png.length + '\n\n');
			client.writeBytes(png);
			client.flush();
			client.close();
			client = null;
		}
	}
}