package hdctl.net
{
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.ProgressEvent;
	import flash.net.Socket;
	import flash.utils.clearInterval;
	import flash.utils.setInterval;
	
	[Event(name="close", type="flash.events.Event")]
	
	public class VirtualController extends EventDispatcher
	{
		private var program:Object;
		private var server:Object;
		private var clients:Array = [];
		private var items:Object = {};
		private var interval:uint;

		public function VirtualController(program:Object, port:uint = 5555)
		{
			this.program = program;

			var ServerSocket:Class = program.getDefinition('flash.net::ServerSocket');
			server = new ServerSocket;
			server.bind(port);
			if (server.bound) 
			{
				log('bound to port ' + port + ' on ' + server.localAddress);
				server.listen();
				if (server.listening)
				{
					log('listening for incoming connections');
					server.addEventListener(Event.CONNECT, client_connect); 
					server.addEventListener(Event.CLOSE, server_close);
				}
			}
		}

		public function reportEntity (report_name:String, access_name:String, readwrite:Boolean = false) : void
		{
			var v:Number;
			try {
				v = program[access_name];
			} catch (err:Error) {
				log('addReportItem: ' + access_name + ' is not aceessible');
				throw err;
			}
			if (readwrite)
			{
				try {
					program[access_name] = v;
				} catch (err:Error) {
					log('addReportItem: ' + access_name + ' is not writeable');
					throw err;
				}
			}
			items[report_name] = { report_name:report_name, access_name:access_name, type:readwrite };
		}
		
		public function set reportInterval (ms:uint) : void
		{
			clearInterval(interval);
			if (ms) setInterval(report, ms);
		}

		public function close () : void
		{
			for each (var client:Socket in clients)
			{
				client.removeEventListener(Event.CLOSE, client_close);
				client.close();
			}
			clearInterval(interval);
			server.removeEventListener(Event.CONNECT, client_connect); 
			server.removeEventListener(Event.CLOSE, server_close); 
			server.close();
		}

		private function client_connect(event:Object) : void
		{
			var client:Object = event.socket;
			client.addEventListener(Event.CLOSE, client_close);
			client.addEventListener(ProgressEvent.SOCKET_DATA, client_data);
			clients.push(client);
			log('client connection from ' + client.remoteAddress + ':' + client.remotePort);
		}
		
		private function client_data(event:Event):void
		{
			var client:Socket = event.target as Socket;
			var data:Array = client.readUTFBytes(client.bytesAvailable).match(/^\s*(\w+)=([\d\.\-]+)/);
			if (!data || !items[data[1]] || !items[data[1]].type && !isNaN(data[2])) return;
			try {
				program[items[data[1]].access_name] = data[2];
			} catch (err:Error) {
				log('clent_data', err);
			}
				
		}
		
		private function client_close(event:Event):void
		{
			var client:Object = clients.splice(clients.indexOf(event.target), 1);
			client.removeEventListener(Event.CLOSE, client_close);
			log('client connection terminated ' + client.remoteAddress + ':' + client.remotePort);
		}
		
		private function server_close (event:Event):void
		{
			server.removeEventListener(Event.CONNECT, client_connect); 
			server.removeEventListener(Event.CLOSE, server_close); 
			log('socket closed by OS');
			dispatchEvent(new Event(Event.CLOSE));
		}
		
		private function log(...args:Array) : void
		{
			program.print('VirtualController: ' + args);
		}
		
		private function report () : void
		{
			var rep:String = '{';
			for each (var item:Object in items) rep += item.report_name + (item.type ? '>' : '<') + Number(program[item.access_name]).toFixed(3) + ' ';
			rep += '}';
			for each (var client:Socket in clients)
			{
				client.writeUTFBytes(rep);
				client.flush();
			}
		}
	}
}